@@ -1,44 +1,54 @@
-// C functions used in 
-// do not consider censoring group now
-// the correpsponding R wrap function is 'crrs.r'
+// reproduce functions in cmprsk using C
+// for simplicity, do not consider censoring group now
+// the correpsponding R wrap function is 'crrc.r'
 
 // author: Bingqing Zhou
-// Date:   04/22/2010
+// Date:   12/21/2008
+// Modified: 06/02/2013
 
 #include <R.h>
 #include <math.h>
-
-// obtain the variance estimate for highly stratified data
+#include <stdlib.h>
+#define LEN sizeof(double)
+// obtain the variance for highly stratified data
 
 void crrvvh(double *t2, int *ici, int *nin, double *x, int *ncov,double *x2, int *ncov2,
       double *ft, int *ndfin, int *strata, int *nstrata, double *wt, double *b, double *v, double *v2)
 {
     const int n1=ncov[0], n2=ncov2[0], n=nin[0], ndf=ndfin[0], np=n1+n2, ns=nstrata[0];
-	int i,j, k, j1, j2, s, count=0, count2=0, pi[n];
-    double a[n][n1],aa[n][n2],tt[ndf][n2], z[np],zb, wye, wyez[np], ss0[n], ss1[n][np], ss2[n][np][np],
-		eta[n][np], vt[np][np], q[n][np], xi[ns][np];
+	int i,j, k, j1, j2, s, count=0, count1=0, count2=0, pi[n];
+    double z[np],zb, wye, wyez[np], ss0[n],*a,*aa,*tt,*ss1,*ss2,*eta,*q,*vt,*xi;
+	a=(double*)malloc(n*n1*LEN);
+	aa=(double*)malloc(n*n2*LEN);
+	tt=(double*)malloc(ndf*n2*LEN);
+	ss1=(double*)malloc(n*np*LEN);
+	ss2=(double*)malloc(n*np*np*LEN);
+	eta=(double*)malloc(n*np*LEN);
+	q=(double*)malloc(n*np*LEN);
+	vt=(double*)malloc(np*np*LEN);
+	xi=(double*)malloc(ns*np*LEN);
 
 	//relates the matrices to the vector parameters
     if (n1 > 0)
         for (i = 0; i < n; i++)
             for (j = 0; j < n1; j++)
-                a[i][j]= x[i + n * j];
+                *(a+i*n1+j)= x[i + n * j];
 
     if (n2 > 0)
         for (i = 0; i < n; i++)
             for (j = 0; j < n2; j++)
-                aa[i][j]= x2[i + n * j];
+                *(aa+i*n2+j)= x2[i + n * j];
     if (n2 > 0)
         for (i = 0; i < ndf; i++)
             for (j = 0; j < n2; j++)
-                tt[i][j]= ft[i + ndf * j];
+                *(tt+i*n2+j)= ft[i + ndf * j];
 
     // initialization
 	for (i = 0; i < n; i++)
 	{
 		pi[i] = 0;
 		for (j = 0; j < np; j++)
-			eta[i][j] = q[i][j] = 0;
+			*(eta+i*np+j) = *(q+i*np+j) = 0;
 	}
 
 	for (i = 0; i < n; i++)
@@ -46,19 +56,19 @@ void crrvvh(double *t2, int *ici, int *nin, double *x, int *ncov,double *x2, int
         ss0[i] = 0;
         for (j = 0; j < np; j++)
         {
-            ss1[i][j]= 0;
+            *(ss1+i*np+j)= 0;
             for (k = 0; k < np; k++)
-                ss2[i][j][k] = 0;
+                *(ss2+i*np*np+j*np+k) = 0;
         }
     }
 
 	for (i = 0; i < np; i++)
         for (j = 0; j < np; j++)
-			vt[i][j] = 0;
+			*(vt+i*np+j) = 0;
 
 	for (i = 0; i < ns; i++)
         for (j = 0; j < np; j++)
-			xi[i][j] = 0;
+			*(xi+i*np+j) = 0;
 
     // obtain s(0)(beta,t),s(1)(beta,t), and s(2)(beta,t) at all type 1 failure times
     for (i = 0; i < n; i++)
@@ -78,15 +88,15 @@ void crrvvh(double *t2, int *ici, int *nin, double *x, int *ncov,double *x2, int
             if (n1 > 0)
                 for (j1 = 0; j1 < n1; j1 ++)
                 {
-                     z[j1] = a[j][j1];
-                     zb += b[j1] * a[j][j1];
+                    z[j1] = *(a+j*n1+j1);
+                    zb += b[j1] * (*(a+j*n1+j1));
                 }
 
             if (n2 > 0)
                 for (k = 0; k < n2; k++)
                 {
-                     z[n1+k] = aa[j][k] * tt[count][k];
-                     zb += b[n1+k] * aa[j][k] * tt[count][k];
+                    z[n1+k] = (*(aa+j*n2+k)) * (*(tt+count*n2+k));
+                    zb += b[n1+k] * (*(aa+j*n2+k)) * (*(tt+count*n2+k));
                 }
 
             if (t2[j] >= t2[i])
@@ -97,13 +107,13 @@ void crrvvh(double *t2, int *ici, int *nin, double *x, int *ncov,double *x2, int
             ss0[i] += wye;
             for (k = 0; k < np; k++)
             {
-                ss1[i][k] += wye * z[k];
+                *(ss1+i*np+k) += wye * z[k];
                 for (j1 = 0; j1 < np; j1++)
-                    ss2[i][k][j1] += wye * z[k] * z[j1];
+                    *(ss2+i*np*np+k*np+j1) += wye * z[k] * z[j1];
             }
         }
         count++;
-	}
+    }
 
   ///////////////////////////////////////////////////////////////
   
@@ -112,7 +122,7 @@ void crrvvh(double *t2, int *ici, int *nin, double *x, int *ncov,double *x2, int
     for (i = 0; i < n; i++)
     {
         for (k = 0; k < np; k++)
-            eta[i][k] = 0;
+            *(eta+i*np+k) = 0;
 
         //dN_i portion of eta_i
         if (ici[i] == 1) 
@@ -123,23 +133,23 @@ void crrvvh(double *t2, int *ici, int *nin, double *x, int *ncov,double *x2, int
 			if (n1 > 0)
 				for (j = 0; j < n1; j++)
 				{
-					z[j] = a[i][j];
-					zb += b[j] * a[i][j];
+					z[j] = *(a+i*n1+j);
+					zb += b[j] * (*(a+i*n1+j));
 				}
 			if (n2 > 0)
 				for (k = 0; k < n2; k++)
 				{
-					z[n1+k]= aa[i][k] * tt[count][k];
-					zb += b[n1+k] * aa[i][k] * tt[count][k];
+					z[n1+k]= (*(aa+i*n2+k))* (*(tt+count*n2+k));
+					zb += b[n1+k] * (*(aa+i*n2+k))* (*(tt+count*n2+k));
 				}
       
 			for (k = 0; k < np; k++)
-				eta[i][k] = z[k] - ss1[i][k]/ss0[i];
+				*(eta+i*np+k)= z[k] - (*(ss1+i*np+k))/ss0[i];
 
 			//information
 			for (j = 0; j < np; j++)
 				for (k = 0; k < np; k++)
-					vt[j][k] += ss2[i][j][k]/ss0[i] - ss1[i][j] * ss1[i][k]/ss0[i]/ss0[i];
+					*(vt+j*np+k)+= (*(ss2+i*np*np+j*np+k))/ss0[i] - (*(ss1+i*np+j))* (*(ss1+i*np+k))/ss0[i]/ss0[i];
 
 			count++;
 		}
@@ -173,14 +183,14 @@ void crrvvh(double *t2, int *ici, int *nin, double *x, int *ncov,double *x2, int
 					if (n1 > 0)
 						for (j = 0; j < n1; j++)
 						{
-							z[j] = a[j2][j];
-							zb += b[j] * a[j2][j];
+							z[j] = *(a+j2*n1+j);
+							zb += b[j] *(*(a+j2*n1+j));
 						}
 					if (n2 > 0)
 						for (k = 0; k < n2; k++)
 						{
-							z[n1+k]= aa[j2][k] * tt[count2-1][k];
-							zb += b[n1+k] * aa[j2][k] * tt[count2-1][k];
+							z[n1+k]= (*(aa+j2*n2+k)) * (*(tt+(count2-1)*n2+k));
+							zb += b[n1+k] * (*(aa+j2*n2+k)) * (*(tt+(count2-1)*n2+k));
 						}
 
 					wye += exp(zb)* wt[j1] / wt[j2];
@@ -189,9 +199,9 @@ void crrvvh(double *t2, int *ici, int *nin, double *x, int *ncov,double *x2, int
 				}
 
 				for (k = 0; k < np; k++)
-					q[i][k] += (wyez[k] - ss1[j1][k]/ss0[j1] * wye) /ss0[j1];
+					*(q+i*np+k) += (wyez[k] - (*(ss1+j1*np+k))/ss0[j1] * wye) /ss0[j1];
 			}
-		}
+		}  
 	}
 
   ///////////////////////////////////////////////////////////////
@@ -199,18 +209,18 @@ void crrvvh(double *t2, int *ici, int *nin, double *x, int *ncov,double *x2, int
 	for (i = 0; i < n; i++)
         for (k = 0; k < np; k++)
 		{
-			if (ici[i]==0) eta[i][k] += q[i][k]/pi[i];
+			if (ici[i]==0) *(eta+i*np+k) += (*(q+i*np+k))/pi[i];
 		    for (j = 0; j < n; j++)
 			{
 				    if (t2[j] > t2[i]) break;
-				    if (ici[j] == 0) eta[i][k] -= q[j][k]/pi[j]/pi[j];
+				    if (ici[j] == 0) *(eta+i*np+k)-= (*(q+j*np+k))/pi[j]/pi[j];
 			 }
 		}
 
 	//output information
     for (j = 0; j < np; j++)
         for (k= 0; k < np; k++)
-            v[j  + k * np] = vt[j][k] ;   
+            v[j  + k * np] = *(vt+j*np+k) ;   
 
 	//output variance of the estimating equation
 	for (i = 0; i < np*np; i++)
@@ -220,12 +230,14 @@ void crrvvh(double *t2, int *ici, int *nin, double *x, int *ncov,double *x2, int
 		{
 			if (strata[j] != i+1) continue;
 			for (k = 0; k < np; k++)
-				xi[i][k] += eta[j][k];
+				*(xi+i*np+k)+= *(eta+j*np+k);
 		}
 	for (j = 0; j < np; j++)
 		for (k= 0; k < np; k++)
 			for (i = 0; i < ns; i++)
-				v2[j  + k * np] += xi[i][j] * xi[i][k];
+				v2[j  + k * np] += (*(xi+i*np+j)) * (*(xi+i*np+k));
+
+	free(a);free(aa);free(tt);free(ss1);free(ss2);free(eta);free(q);free(vt);free(xi);
 }
 
 
@@ -238,21 +250,24 @@ void crrf(double *t2, int *ici, int *nin, double *x, int *ncov,  double *x2, int
 {
     int i,j, k, j1,  count=0;
 	const int n1=ncov[0], n2=ncov2[0], n=nin[0], ndf=ndfin[0];
-	double a[n][n1], aa[n][n2], tt[ndf][n2], likli=0, ss0,zb;
-	
+	double likli=0, ss0,zb,*a,*aa,*tt;
+	a=(double*)malloc(n*n1*LEN);
+	aa=(double*)malloc(n*n2*LEN);
+	tt=(double*)malloc(ndf*n2*LEN);
+ 
 	if (n1 > 0)
         for (i = 0; i < n; i++)
             for (j = 0; j < n1; j++)
-                a[i][j]= x[i + n * j];
+                *(a+i*n1+j)= x[i + n * j];
 
     if (n2 > 0)
         for (i = 0; i < n; i++)
             for (j = 0; j < n2; j++)
-                aa[i][j]= x2[i + n * j];
+                *(aa+i*n2+j)= x2[i + n * j];
     if (n2 > 0)
         for (i = 0; i < ndf; i++)
             for (j = 0; j < n2; j++)
-                tt[i][j]= ft[i + ndf * j];
+                *(tt+i*n2+j)= ft[i + ndf * j];
 
     for (i = 0; i < n; i++)
 	{
@@ -261,10 +276,10 @@ void crrf(double *t2, int *ici, int *nin, double *x, int *ncov,  double *x2, int
 		//first part of logliklihood
 		if (n1 > 0)
 			for (j = 0; j < n1; j++)
-				likli += b[j] * a[i][j];
+				likli += b[j] * (*(a+i*n1+j));
 		if (n2 > 0)
 			for (k = 0; k < n2; k++)
-				likli += b[n1+k] * aa[i][k] * tt[count][k];
+				likli += b[n1+k] *(*(aa+i*n2+k))* (*(tt+count*n2+k));
         //second part of logliklihood
 		ss0=0;
 		for (j = 0; j < n; j++)
@@ -273,10 +288,10 @@ void crrf(double *t2, int *ici, int *nin, double *x, int *ncov,  double *x2, int
 			zb = 0.0;
 			if (n1 > 0)
 				for (j1 = 0; j1 < n1; j1 ++)
-					zb += b[j1] * a[j][j1];
+					zb += b[j1] *(*(a+j*n1+j1));
 			if (n2 > 0)
 				for (k = 0; k < n2; k++)
-					zb += b[n1+k] * aa[j][k] * tt[count][k];
+					zb += b[n1+k] * (*(aa+j*n2+k)) * (*(tt+count*n2+k));
 
 			if (t2[j] >= t2[i]) 
 				ss0 += exp(zb);
@@ -293,6 +308,7 @@ void crrf(double *t2, int *ici, int *nin, double *x, int *ncov,  double *x2, int
    
 	*lik=likli;
 
+	free(a);free(aa);free(tt);
 }
 
 
@@ -303,28 +319,32 @@ void crrfsv(double *t2, int *ici, int *nin, double *x, int *ncov,  double *x2, i
 {
     int i,j, k, j1,  count=0;
 	const int n1=ncov[0], n2=ncov2[0], n=nin[0], ndf=ndfin[0], np=n1+n2;
-	double a[n][n1],aa[n][n2],tt[ndf][n2],likli=0,ss0,ss1[np],ss2[np][np],z[np],zb,wye,vt[np][np];
-
+	double likli=0,ss0,ss1[np],z[np],zb,wye,*a,*aa,*tt,*ss2,*vt;
+	a=(double*)malloc(n*n1*LEN);
+	aa=(double*)malloc(n*n2*LEN);
+	tt=(double*)malloc(ndf*n2*LEN);
+    ss2=(double*)malloc(np*np*LEN);
+    vt=(double*)malloc(np*np*LEN);
     for (i = 0; i < np; i++)
 	{
 		st[i] = 0;
 		for (j = 0; j < np; j++)
-			vt[i][j]= 0;
+			*(vt+i*np+j)= 0;
 	}
 
 	if (n1 > 0)
 		for (i = 0; i < n; i++)
 			for (j = 0; j < n1; j++)
-				a[i][j]= x[i + n * j];
+				*(a+i*n1+j)= x[i + n * j];
 
     if (n2 > 0)
 		for (i = 0; i < n; i++)
 		    for (j = 0; j < n2; j++)
-			    aa[i][j]= x2[i + n * j];
+			    *(aa+i*n2+j)= x2[i + n * j];
     if (n2 > 0)
 		for (i = 0; i < ndf; i++)
 		    for (j = 0; j < n2; j++)   
-		     	tt[i][j]= ft[i + ndf * j];
+		     	*(tt+i*n2+j)= ft[i + ndf * j];
 
 
     for (i = 0; i < n; i++)
@@ -336,14 +356,14 @@ void crrfsv(double *t2, int *ici, int *nin, double *x, int *ncov,  double *x2, i
 		if (n1 > 0)
 			for (j = 0; j < n1; j++)
 			{
-				st[j]  +=  a[i][j];
-				zb += b[j] * a[i][j];
+				st[j]  +=  *(a+i*n1+j);
+				zb += b[j] *(*(a+i*n1+j));
 			}
 		if (n2 > 0)
 			for (k = 0; k < n2; k++)
 			{
-				st[n1+k]  +=  aa[i][k] * tt[count][k];
-				zb += b[n1+k] * aa[i][k] * tt[count][k];
+				st[n1+k]  +=  (*(aa+i*n2+k)) * (*(tt+count*n2+k));
+				zb += b[n1+k] * (*(aa+i*n2+k)) * (*(tt+count*n2+k));
 			}
 
         likli += zb;
@@ -354,7 +374,7 @@ void crrfsv(double *t2, int *ici, int *nin, double *x, int *ncov,  double *x2, i
 		{
 			ss1[k] = 0;
 			for (j1 = 0; j1 < np; j1 ++)
-				ss2[k][j1] = 0;
+				*(ss2+k*np+j1) = 0;
 		}
 
 		for (j = 0; j < n; j++)
@@ -368,15 +388,15 @@ void crrfsv(double *t2, int *ici, int *nin, double *x, int *ncov,  double *x2, i
 			if (n1 > 0)
 				for (j1 = 0; j1 < n1; j1 ++)
 				{
-					z[j1] = a[j][j1];
-					zb += b[j1] * a[j][j1];
+					z[j1] = *(a+j*n1+j1);
+					zb += b[j1] * (*(a+j*n1+j1));
 				}
 				    
 			if (n2 > 0)
 				for (k = 0; k < n2; k++)
 				{
-					z[n1+k] = aa[j][k] * tt[count][k];
-					zb += b[n1+k] * aa[j][k] * tt[count][k];
+					z[n1+k] = (*(aa+j*n2+k)) * (*(tt+count*n2+k));
+					zb += b[n1+k] * (*(aa+j*n2+k)) * (*(tt+count*n2+k));
 				}
 
 
@@ -390,7 +410,7 @@ void crrfsv(double *t2, int *ici, int *nin, double *x, int *ncov,  double *x2, i
 			{
 				ss1[k] += wye * z[k];
 				for (j1 = 0; j1 < np; j1++)
-					ss2[k][j1] += wye * z[k] * z[j1];
+					*(ss2+k*np+j1) += wye * z[k] * z[j1];
 			}
 
 		}
@@ -401,7 +421,7 @@ void crrfsv(double *t2, int *ici, int *nin, double *x, int *ncov,  double *x2, i
 		{
 			st[k] -= ss1[k] / ss0;
 			for (j1 = 0; j1 < np; j1++)
-				vt[k][j1] += ss2[k][j1]/ss0 - ss1[k] * ss1[j1]/ss0/ss0;
+				*(vt+k*np+j1)+= (*(ss2+k*np+j1))/ss0 - ss1[k] * ss1[j1]/ss0/ss0;
 		}
 		
 	    count++;
@@ -411,8 +431,9 @@ void crrfsv(double *t2, int *ici, int *nin, double *x, int *ncov,  double *x2, i
 
     for (i = 0; i < np; i++)
 		for (j = 0; j < np; j++)
-			v[i + np * j] = vt[i][j];
+			v[i + np * j] = *(vt+i*np+j);
 
+	free(a);free(aa);free(tt);free(ss2);free(vt);
 }
 
 
@@ -426,15 +447,23 @@ void crrvv(double *t2, int *ici, int *nin, double *x, int *ncov,double *x2, int
 {
     const int n1=ncov[0], n2=ncov2[0], n=nin[0], ndf=ndfin[0], np=n1+n2;
 	int i,j, k, j1, j2,count=0, count1=0, count2=0, pi[n];
-    double a[n][n1],aa[n][n2],tt[ndf][n2], z[np],zb, wye, wyez[np], ss0[n], ss1[n][np], ss2[n][np][np],
-		eta[n][np], vt[np][np], q[n][np];
+    double z[np],zb, wye, wyez[np], ss0[n],*a,*aa,*tt,*ss1,*ss2,*eta,*vt,*q;
+		
+	a=(double*)malloc(n*n1*LEN);
+	aa=(double*)malloc(n*n2*LEN);
+	tt=(double*)malloc(ndf*n2*LEN);
+	ss1=(double*)malloc(n*np*LEN);
+	ss2=(double*)malloc(n*np*np*LEN);
+	eta=(double*)malloc(n*np*LEN);
+	vt=(double*)malloc(np*np*LEN);
+	q=(double*)malloc(n*np*LEN);
 
     // initialization
 	for (i = 0; i < n; i++)
 	{
 		pi[i] = 0;
 		for (j = 0; j < np; j++)
-			eta[i][j] = q[i][j] = 0;
+			*(eta+i*np+j) = *(q+i*np+j) = 0;
 	}
 
 	for (i = 0; i < n; i++)
@@ -442,29 +471,29 @@ void crrvv(double *t2, int *ici, int *nin, double *x, int *ncov,double *x2, int
         ss0[i] = 0;
         for (j = 0; j < np; j++)
         {
-            ss1[i][j]= 0;
+            *(ss1+i*np+j)= 0;
             for (k = 0; k < np; k++)
-                ss2[i][j][k] = 0;
+                *(ss2+i*np*np+j*np+k) = 0;
         }
     }
 
 	for (i = 0; i < np; i++)
         for (j = 0; j < np; j++)
-			vt[i][j] = 0;
+			*(vt+i*np+j) = 0;
 
     if (n1 > 0)
         for (i = 0; i < n; i++)
             for (j = 0; j < n1; j++)
-                a[i][j]= x[i + n * j];
+                *(a+i*n1+j)= x[i + n * j];
 
     if (n2 > 0)
         for (i = 0; i < n; i++)
             for (j = 0; j < n2; j++)
-                aa[i][j]= x2[i + n * j];
+                *(aa+i*n2+j)= x2[i + n * j];
     if (n2 > 0)
         for (i = 0; i < ndf; i++)
             for (j = 0; j < n2; j++)
-                tt[i][j]= ft[i + ndf * j];
+                *(tt+i*n2+j)= ft[i + ndf * j];
 
     // obtain s(0)(beta,t),s(1)(beta,t), and s(2)(beta,t) at all type 1 failure times
     for (i = 0; i < n; i++)
@@ -482,15 +511,15 @@ void crrvv(double *t2, int *ici, int *nin, double *x, int *ncov,double *x2, int
             if (n1 > 0)
                 for (j1 = 0; j1 < n1; j1 ++)
                 {
-                    z[j1] = a[j][j1];
-                    zb += b[j1] * a[j][j1];
+                    z[j1] = *(a+j*n1+j1);
+                    zb += b[j1] * (*(a+j*n1+j1));
                 }
 
             if (n2 > 0)
                 for (k = 0; k < n2; k++)
                 {
-                    z[n1+k] = aa[j][k] * tt[count][k];
-                    zb += b[n1+k] * aa[j][k] * tt[count][k];
+                    z[n1+k] = (*(aa+j*n2+k))*(*(tt+count*n2+k));
+                    zb += b[n1+k] * (*(aa+j*n2+k))*(*(tt+count*n2+k));
                 }
 
             if (t2[j] >= t2[i])
@@ -501,9 +530,9 @@ void crrvv(double *t2, int *ici, int *nin, double *x, int *ncov,double *x2, int
             ss0[i] += wye;
             for (k = 0; k < np; k++)
             {
-                ss1[i][k] += wye * z[k];
+                *(ss1+i*np+k) += wye * z[k];
                 for (j1 = 0; j1 < np; j1++)
-                    ss2[i][k][j1] += wye * z[k] * z[j1];
+                    *(ss2+i*np*np+k*np+j1) += wye * z[k] * z[j1];
             }
         }
         count++;
@@ -516,7 +545,7 @@ void crrvv(double *t2, int *ici, int *nin, double *x, int *ncov,double *x2, int
     for (i = 0; i < n; i++)
     {
         for (k = 0; k < np; k++)
-            eta[i][k] = 0;
+            *(eta+i*np+k) = 0;
 
 		count1 = 0;
         //dLamda portion of eta_i
@@ -531,15 +560,15 @@ void crrvv(double *t2, int *ici, int *nin, double *x, int *ncov,double *x2, int
             if (n1 > 0)
                 for (j1 = 0; j1 < n1; j1 ++)
                 {
-                    z[j1] = a[i][j1];
-                    zb += b[j1] * a[i][j1];
+                    z[j1] = *(a+i*n1+j1);
+                    zb += b[j1] * (*(a+i*n1+j1));
                 }
 
             if (n2 > 0)
                 for (k = 0; k < n2; k++)
                 {
-                    z[n1+k] = aa[i][k] * tt[count1][k];
-                    zb += b[n1+k] * aa[i][k] * tt[count1][k];
+                    z[n1+k] = (*(aa+i*n2+k))*(*(tt+count1*n2+k));
+                    zb += b[n1+k] * (*(aa+i*n2+k))*(*(tt+count1*n2+k));
                 }
 
 
@@ -550,7 +579,7 @@ void crrvv(double *t2, int *ici, int *nin, double *x, int *ncov,double *x2, int
             else continue;
 
             for (k = 0; k < np; k++)
-                eta[i][k] -= (z[k] - ss1[j][k]/ss0[j]) * wye / ss0[j];
+                *(eta+i*np+k) -= (z[k] - (*(ss1+j*np+k))/ss0[j]) * wye / ss0[j];
 
             count1 ++;
         }
@@ -564,23 +593,23 @@ void crrvv(double *t2, int *ici, int *nin, double *x, int *ncov,double *x2, int
 			if (n1 > 0)
 				for (j = 0; j < n1; j++)
 				{
-					z[j] = a[i][j];
-					zb += b[j] * a[i][j];
+					z[j] = *(a+i*n1+j);
+					zb += b[j] * (*(a+i*n1+j));
 				}
 			if (n2 > 0)
 				for (k = 0; k < n2; k++)
 				{
-					z[n1+k]= aa[i][k] * tt[count][k];
-					zb += b[n1+k] * aa[i][k] * tt[count][k];
+					z[n1+k]=  (*(aa+i*n2+k))*(*(tt+count*n2+k));
+					zb += b[n1+k] * (*(aa+i*n2+k))*(*(tt+count*n2+k));
 				}
 
 			for (k = 0; k < np; k++)
-				eta[i][k] += z[k] - ss1[i][k]/ss0[i];
+				*(eta+i*np+k) += z[k] - (*(ss1+i*np+k))/ss0[i];
 
 			//information
 			for (j = 0; j < np; j++)
 				for (k = 0; k < np; k++)
-					vt[j][k] += ss2[i][j][k]/ss0[i] - ss1[i][j] * ss1[i][k]/ss0[i]/ss0[i];
+					*(vt+j*np+k) += (*(ss2+i*np*np+j*np+k))/ss0[i] - (*(ss1+i*np+j)) * (*(ss1+i*np+k))/ss0[i]/ss0[i];
 
 			count++;
 		}
@@ -614,14 +643,14 @@ void crrvv(double *t2, int *ici, int *nin, double *x, int *ncov,double *x2, int
 					if (n1 > 0)
 						for (j = 0; j < n1; j++)
 						{
-							z[j] = a[j2][j];
-							zb += b[j] * a[j2][j];
+							z[j] = *(a+j2*n1+j) ;
+							zb += b[j] * (*(a+j2*n1+j));
 						}
 					if (n2 > 0)
 						for (k = 0; k < n2; k++)
 						{
-							z[n1+k]= aa[j2][k] * tt[count2-1][k];
-							zb += b[n1+k] * aa[j2][k] * tt[count2-1][k];
+							z[n1+k]= (*(aa+j2*n2+k))*(*(tt+(count2-1)*n2+k));
+							zb += b[n1+k] * (*(aa+j2*n2+k))*(*(tt+(count2-1)*n2+k));
 						}
 
 					wye += exp(zb)* wt[j1] / wt[j2];
@@ -630,7 +659,7 @@ void crrvv(double *t2, int *ici, int *nin, double *x, int *ncov,double *x2, int
 				}
 
 				for (k = 0; k < np; k++)
-					q[i][k] += (wyez[k] - ss1[j1][k]/ss0[j1] * wye) /ss0[j1];
+					*(q+i*np+k)+= (wyez[k] - (*(ss1+j1*np+k))/ss0[j1] * wye) /ss0[j1];
 			}
 		}
 	}
@@ -640,17 +669,17 @@ void crrvv(double *t2, int *ici, int *nin, double *x, int *ncov,double *x2, int
 	for (i = 0; i < n; i++)
         for (k = 0; k < np; k++)
 		{
-			if (ici[i]==0) eta[i][k] += q[i][k]/pi[i];
+			if (ici[i]==0) *(eta+i*np+k)+= *(q+i*np+k)/pi[i];
 		    for (j = 0; j < n; j++)
 			{
 				    if (t2[j] > t2[i]) break;
-				    if (ici[j] == 0) eta[i][k] -= q[j][k]/pi[j]/pi[j];
+				    if (ici[j] == 0) *(eta+i*np+k)-= (*(q+j*np+k))/pi[j]/pi[j];
 			 }
 		}
 
     for (j = 0; j < np; j++)
         for (k= 0; k < np; k++)
-            v[j  + k * np] = vt[j][k] ;   
+            v[j  + k * np] = *(vt+j*np+k) ;   
 
 	for (i=0; i<np*np; i++)
 		v2[i]=0;
@@ -658,106 +687,10 @@ void crrvv(double *t2, int *ici, int *nin, double *x, int *ncov,double *x2, int
     for (j = 0; j < np; j++)
         for (k= 0; k < np; k++)
 			for (i = 0; i < n; i++)
-                v2[j  + k * np] += eta[i][j] * eta[i][k] ;
-
+                v2[j  + k * np] += (*(eta+i*np+j))* (*(eta+i*np+k));
+free(a);free(aa);free(tt);free(ss1);free(ss2);free(eta);free(vt);free(q);
 }
 
 
 //end of the functions
 
-// example
-/*
-
-setwd( "C:/fine/practice")
-dyn.load("crrc.dll")
-library(cmprsk)
-set.seed(1671)
-n=200
-np1=3
-ftime <- sort(rexp(n))
-fstatus <- sample(0:2,n,replace=TRUE)
-cov <- matrix(runif(np1*n),nrow=n)
-cov2 = cbind(cov[,1],cov[,1])
-np = np1 + ncol(cov2)
-npt = ncol(cov2)
-uft <- ftime[fstatus==1]
-tfs=cbind(uft,uft^2)
-ndf=length(uft)
-b= c(-.2,.7,1,-.5,3)
-cengroup = rep(1,length(fstatus))
-ncg = 1
-cenind <- ifelse(fstatus==1,1,0)
-uuu <- matrix(0,nrow=ncg,ncol=length(ftime))
-
-for (k in 1:ncg) {
-    u <- do.call('survfit',list(formula=Surv(ftime,cenind)~1,data=data.frame(ftime,cenind,cengroup),subset=cengroup==k))
-    ### note: want censring dist km at ftime-
-    u <- summary(u,times=sort(ftime*(1-.Machine$double.eps)))
-    uuu[k,1:length(u$surv)] <- u$surv
-}
-
-  #####################
-  start1 <- Sys.time()
-.C("crrf", as.double(ftime),as.integer(fstatus),as.integer(length(ftime)),
-   as.double(cov), as.integer(np-npt), as.double(cov2),as.integer(npt),
-   as.double(tfs),as.integer(ndf), as.double(uuu), 
-   as.double(b), double(1))[[12]]
-start2 <- Sys.time()
-.Fortran('crrf',as.double(ftime),as.integer(fstatus),
-                  as.integer(length(ftime)),as.double(cov),as.integer(np-npt),
-                  as.integer(np),as.double(cov2),as.integer(npt),
-                  as.double(tfs),as.integer(ndf),as.double(uuu),
-                  as.integer(ncg),as.integer(cengroup),as.double(b),
-                  double(1),double(np),package="cmprsk")[15:16]
-end2 <- Sys.time()
-start2-start1
-end2-start2
-
-  ####################
-
-    start1 = Sys.time()
-  .C("crrfsv", as.double(ftime),as.integer(fstatus),as.integer(length(ftime)),
-   as.double(cov), as.integer(np-npt), as.double(cov2),as.integer(npt),
-   as.double(tfs),as.integer(ndf), as.double(uuu), 
-   as.double(b), double(1), double(np), double(np*np))[12:14]
-
-  start2 = Sys.time()
-  .Fortran('crrfsv',as.double(ftime),as.integer(fstatus),
-                  as.integer(length(ftime)),as.double(cov),as.integer(np-npt),
-                  as.integer(np),as.double(cov2),as.integer(npt),
-                  as.double(tfs),as.integer(ndf),as.double(uuu),
-                  as.integer(ncg),as.integer(cengroup),as.double(b),
-                  double(1),double(np),double(np*np),double(np),double(np),
-                  double(np*np))[15:17]
-   end = Sys.time()
-    
-   start2-start1
-   end-start2
-  
-	#############################
-
-
-  start <- Sys.time()
-.C("crrvv", as.double(ftime),as.integer(fstatus),as.integer(length(ftime)),
-   as.double(cov), as.integer(np-npt), as.double(cov2),as.integer(npt),as.double(tfs),
-   as.integer(ndf), as.double(uuu),  as.double(b), double(np*np), double(np*np))[12:13]
-  start1 <- Sys.time()
-  
- # uft <- unique(ftime[fstatus==1])
- # tfs=cbind(uft,uft^2)
- # ndf=length(uft)
-   .Fortran('crrvv',as.double(ftime),as.integer(fstatus),
-                as.integer(length(ftime)),as.double(cov),as.integer(np-npt),
-                as.integer(np),as.double(cov2),as.integer(npt),
-                as.double(tfs),as.integer(ndf),as.double(uuu),
-                as.integer(ncg),as.integer(cengroup),as.double(b),
-                double(np*np),double(np*np),double(np*np),
-                double(length(ftime)*(np+1)),double(np),double(np),
-                double(2*np),double(np),double(ncg*np),integer(ncg), package="cmprsk")[15:16]
-   end <- Sys.time()
-   start1-start
-   end-start1
-*/
-
-
-
